//---------------------------------------------------------------------------
// filename: PC21.CPP
// Version: 0.0.1
// date: 17.05.2001
// suggestion: CompuMotor
// author(s): Jrg Wolters for DOS and Markus Leitz for LINUX
// doc: PC21.DOC
//  comments: These routines make up the core of the C routines supporting the PC21.  They include routines to read characters, write characters, read strings, write strings, read absolute buffered position reports and incremental unbuffered (immediate) position reports.

// pc21.cpp
//
// Copyright 2001 by Markus Leitz
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// version 2 as published by the Free Software Foundation.//
// This library is distributed  WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//-------------- define  Includes -------------------------------------



#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/io.h>
#include "pc21.h"

//---------------------------------------------------------------------------
//	initialize
//     Reset the PC21 board at the designated address
//     (Default: 0x300 or 768 decimal).
//
//     The routing checks to see that the board is ready to accept
//     commands from the user program. If the address is wrong the
//     program will print a message and exit.  Returns 0.
//---------------------------------------------------------------------------
int initialize ()
{
  
  char i;
  int status_addr = DEFAULT_ADDR + 1;
	
  printf("Status_addr : %x \n",status_addr);
  
  if( ioperm(DEFAULT_ADDR,2,1) == 0 )

    { 
       
      printf(" pc21 ioperm  succeed \n");
    }
  else 
    { printf(" pc21 ioperm failed \n");
    }

  
  printf("first outb....now \n");   
  outb ( (CB | BIT2MASK ), status_addr);   // stop BMA timeout, set bit 2
 
 
  printf("second outb ... now\n");
  outb ( (CB & (!BIT2MASK) ), status_addr );// start BMA, clear bit 2

  usleep(100); // wait for BMA timeout, worst case 100ms

 
  printf("third outb ...now\n");
  outb ( (CB & (!FAIL)),status_addr);    // clear bit 5 of control byte

  printf("fourth outb ... now\n");
  outb ((CB | FAIL),status_addr);       // set bit 5 of control byte	

  usleep(500);                              // wait for ready

  printf("first inb ... now\n");  
  i=inb(status_addr);
  
  printf("read out from  status_adr is : %i \n",i);
  if ((i != IDB_M) && (i != CHECKADDR))    // check for reset
    {
        
      return 1;
	
    }
  else
    {
	
      return 0;
    }
}  /* END INITIALIZE */



/*
 * pc21_writech: Writes a single character to the PC21. PC21 commands are
 *              generated by sending multiple characters to it.
 *              Returns the character of the response.  This
 *              routine has no error checking, improving performance
 *              but leaving the potential for the PC21 to hang the
 *              computer.
 *
 */

void pc21_writech (char alpha)
{

  while (!(inb(DEFAULT_ADDR+1) & IDB_M)); // check to see if ready to take a byte
	
  outb (alpha,DEFAULT_ADDR);    // send the character 
 
  outb ( CHAR_READY, DEFAULT_ADDR+1); //  tell the pc21 to look for a character

  while (inb(DEFAULT_ADDR+1) & IDB_M); //  wait for the pc21 to recognize it 

  outb(CB,DEFAULT_ADDR+1);    // clear the control byte 

  while (!(inb(DEFAULT_ADDR+1) & IDB_M)); // wait for pc21 to digest the byte 


}  /* END PC21_WRITECH */

/*
 * pc21_readch: Reads one character of a PC21 response to an information
 *              request.  Returns the character of the response.  This
 *              routine has no error checking, improving performance
 *              but leaving the potential for the PC21 to hang the
 *              computer.
 */

char pc21_readch()
{
	int esc_seq = 0;
	char alpha=0;                      				// character returned

	  while (!(inb(DEFAULT_ADDR+1) & ODB))
	   
	    {   // wait for char to be ready

//  		if (kbhit())
	      //	        {
//  			esc_seq=getch();
//  			if (esc_seq==ESC)
//  				exit(0);
//  		}
  	}

	alpha = inb(DEFAULT_ADDR);          		// get the char
	
        outb (ACK,DEFAULT_ADDR+1);               // tell the pc21 you've got it

	while ((inb(DEFAULT_ADDR+1) & ODB));     // wait for the pc21's acknowledge
	
        outb (CB,DEFAULT_ADDR+1);                //reset the control byte to normal

	return(alpha);                               // return the character read

} /* END PC21_READCH */

/*
 * pc21_writecmd: Writes a command string to the PC21.
 */

void pc21_writecmd(char *s)
{
	while (*s)                             		// increment through string
		 pc21_writech (*s++);               		// until a null is found

} /* END PC21_WRITECMD */

/*
 * pc21_readanswer: Reads a complete PC21 response string.  This routine
 *                  assumes the string will be terminated by a carriage
 *                  routine.  It will hang the program if no CR is
 *                  forthcoming.
 */

char *pc21_readanswer ( char *s)
{
	while ((*s++ = pc21_readch()) != CR)	;
	*s = '\0';
	return s;
} /* END PC21_READANSWER */

/*
 * PRpos():  Returns the current position as a long int.  The routine does
 *	          the alpha to decimal conversion.  It uses the buffered PR
 *           command which will not respond when another command is in
 *           process (such as a move). To get an instant report,
 *           use the W3pos() or W1pos() functions.
 */

long PRpos()
{
	char posreq[] = { " pr " },  	// the position request string sans axis
		 answer[15],   				// a place to put the position
		 buffer[4];    				// a place to put the itoa answer
	char * ap;         				// a pointer to the answer string

	ap = answer + 2;   				// point to the first number of the position
											// report in the command string
	pc21_writecmd( posreq);   		// ask for the position
	pc21_readanswer( answer); 		// get the position

	return atol( ap );  			// convert to a long and return
}  /* END PRPOS */

/*
 * W1pos():  Returns the current position
 *           as a long int.  The routine converts the binary notation
 *           to a long int.  It uses the unbuffered W1 command which
 *           is the fastest reportback, and works any time, even
 *           during buffered commands such as moves.
 */

long W1pos()
{
	char posreq[] = { " w1 " },	// the position request string sans axis
		 buffer[4];              	// a place to put the itoa answer
	char * pos;
	union { long pos;
			char charpos[4]; } binary;
	int i;

	pos = &binary.charpos[3];
											// in the command string
	pc21_writecmd( posreq);       // ask for the position
	for ( i=0; i<4; i++)
		*pos-- = pc21_readch();

	return binary.pos;
}  /* END W1POS */

/*
 * W3pos():  Returns the current position
 *           as a long int.  The routine converts the hex to a long int.
 *           It uses the unbuffered W3 command which will gets a response
 *           regardless of whether the motor is moving.  The only faster
 *           reportback is the binary reportback, called via W1pos().
 */

long W3pos()
{
	long decimalposition;
	char posreq[] = { " w3 " },	// the position request string sans axis
		  buffer[4];               // a place to put the itoa answer
	char W3position[10];
											// in the command string
	pc21_writecmd( posreq);       // ask for the position
	pc21_readanswer( W3position);
	W3position[0] = '0';          // make into hex format
	W3position[1] = 'x';
	if ( sscanf( W3position, "%li", &decimalposition ) > 0 )
		return decimalposition;
	else
	{
		cerr << "\nEs konnte mit dem W3 Kommando nicht gelesen werden.\n"  << endl;
		exit( 1 );
     }
	return decimalposition;
}  /* END W3POS */


/*
 * pc21_stop():  This routine sends a stop command to the PC21.
 *               The stop in the PC21 is a controlled
 *               stop and uses the currently set deceleration/acceleration.
 *               For an immediate stop use the KILL command.
 */

void pc21_stop()
{

	pc21_writecmd( " s ");
}  /* END PC21_STOP */

/*
 * pc21_kill():  This routine sends a kill command to the PC21.
 *               The kill in the PC21 is an immediate
 *               stop and does not use the currently set
 *               deceleration/acceleration.   For a controlled stop
 *               use the STOP command.
 */

void pc21_kill()
{

	pc21_writecmd( " k ");
} /* END PC21_KILL */



