/*
 * Programm Name: pc21.h
 * Version: 0.0.1
 * Date: 17.05.2001
 * Change: 17.05.2001
 * suggestion: CompuMotor (www.compumotor.com)
 * author(s): Jrg Wolters for DOS and Markus Leitz for LINUX
 * doc file : pc21.doc
 * 
 */
//  pc21.h
//
// Copyright 2001 by Markus Leitz
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// version 2 as published by the Free Software Foundation.//
// This library is distributed  WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

#ifndef __PC21_H
#define __PC21_H



#include <sys/io.h>


/*
 * These enums and defines create various STATUS and CONTROL BYTE masks
 * used in handshaking data with the PC21.  Pages 52-53 of the PC21
 * manual (REV -04) describe the functions of each register's bit.
 * Pages 45-50 gives a step by step procedure of the handshake process.
 */


/* Keyboard definitions */

#define	CR           0x0D    // carriage return
#define	ESC          27      // the escape character
#define	FUNKEY       0x0     // function keys are two keys, the first of which is always zero
#define	F1           59
#define	F2           60
#define	F3           61
#define	F4           62
#define	F5           63
#define	F6           64
#define	F7           65
#define	F8           66
#define	F9           67
#define	F10          68

const int TIMEOUT = 132000;  // maximum delay time

enum {
	ACK          =0xE0,
	ALDONE       =0x07,
	ASCIIZERO    =48,
	AXIS_1       =0x01,
	AXIS_2       =0x02,
	AXIS_3       =0x03,
	BFULL        =0x80,    // checks the buffer status - RESERVED
	BIT2MASK     =0x04,    // mask for bit 2 of control byte
	CB           =0x60,    // completes handshakes with status byte
	CB1          =0x61,
	CHAR_READY   =0x70,    // character ready bit
	CHECKADDR    =0x16,    // used to test if board addr set right
	DEFAULT_ADDR =0x300,   // default PC21 address is 768 decimal
	FAIL         =0x20,    // board fail bit
	IDB1         =0x71,
	IDB_M        =0x10,    // input data buffer mask
	INTCLR       =0x20,    // clear interrup bit
	ODB          =0x08,    // output data buffer bit
	RESTART      =0x17,    // byte to restart the pc21
	START        =0x40,    // byte to start the pc21
	START_M      =0x7F,    // start mask
	STOP         =0x64,    // byte to halt the pc21's processor
};

/* Macro definitions */
//#define checkforchar( x )  (inp( x ) & ODB)
//#define IDB_READY          (inp(status_addr) & IDB_M)
//#define ODB_READY          (inp(status_addr) & ODB)

/* check for character in ODB */

inline int checkforchar (char x) {	return (inb(x) & ODB);}

/* check for IDB ready */

inline int IDB_READY(int status_addr) {return (inb(status_addr) & IDB_M);}

/* check for ODB ready */

inline int ODB_READY(int status_addr) {	return (inb(status_addr) & ODB);}


/*
 * This union provides a place to put the status byte.  Its
 * individual bits may then be read out as unique elements.
 * Declare a variable as type status to make use of it.
 */

typedef union status
{
	int Status;
	struct sbit {
		unsigned int unused             : 1;
		unsigned int in_pos             : 1;
		unsigned int load_ready         : 1;
		unsigned int output_ready       : 1;
		unsigned int input_ready        : 1;
		unsigned int board_fail         : 1;
		unsigned int interrupt_active   : 1;
		unsigned int buffer_full        : 1;
	} sbit ;
} status;


/* Prototypes */

void   binwrite( int num_of_axis, int p, unsigned int status_addr); //not implemented

int    getaddress(void);

int    initialize(void);   					

void   pc21_kill();

char * pc21_readanswer( char *s);	

char   pc21_readch();							 void   pc21_stop();

void   pc21_writecmd( char *s);					

void   pc21_writech( char alpha);			    

long   PRpos();										//gets actual position (buffered)
long   W1pos();										//DITO; (unbuffered) bin -> l int
long   W3pos();										//DITO; (unbuffered) hex -> l int


#endif


